// hdf5test.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <gdal.h>
#include <gdal_priv.h>
#include <string>

void AccessHDFData( GDALRasterBand  *poBand );

int _tmain(int argc, _TCHAR* argv[])
{
  GDALAllRegister();

  GDALDataset  *poDataset = (GDALDataset *) GDALOpen( argv[1], GA_ReadOnly );
  if( poDataset == NULL )
  {
    const char * err_msg = CPLGetLastErrorMsg();
    printf("%s", err_msg);
    return -1;
  }

  char** subdatasets = poDataset->GetMetadata("SUBDATASETS");
  std::string subDS(subdatasets[0]);

  std::string temp( subDS.substr( subDS.find("=") + 1, subDS.length() ) );
  GDALDataset *poSubDataset = (GDALDataset *) GDALOpen( temp.c_str(), GA_ReadOnly );

  int rc = poSubDataset->GetRasterCount();
  GDALRasterBand  *poBand = poSubDataset->GetRasterBand( 1 );
  AccessHDFData( poBand );

  GDALClose( poSubDataset );
  GDALClose( poDataset );
  return 0;
}

void AccessHDFData( GDALRasterBand  *poBand )
{
  int nBlockXSize, nBlockYSize;
  poBand->GetBlockSize( &nBlockXSize, &nBlockYSize );

  GDALDataType dataType = poBand->GetRasterDataType();
  int dataTypeSize = GDALGetDataTypeSize( dataType );

  // Read from 0,0
  void *pafScanline = CPLMalloc(dataTypeSize*nBlockXSize*nBlockYSize);

  // Only the first 90000 values are read in the buffer; the block size 
  // of the sample is 300x300 = 90000
  poBand->RasterIO( GF_Read, 0, 0, nBlockXSize, nBlockYSize, 
                    pafScanline, nBlockXSize, nBlockYSize, dataType, 
                    0, 0 );

  CPLFree(pafScanline);

  return;
}
